@@ -1,14 +1,11 @@
+import copy
 from collections import namedtuple
 
-import copy
-import joblib
 import matplotlib
-import matplotlib.pyplot as plt
-import numpy as np
 import pytest
 
 from rnalysis import filtering
-from rnalysis.utils import enrichment_runner, validation
+from rnalysis.utils import enrichment_runner
 from rnalysis.utils.enrichment_runner import *
 from rnalysis.utils.io import *
 from tests import __attr_ref__, __biotype_ref__
@@ -42,7 +39,7 @@ def test_calc_randomization_pval():
     hypergeom_pval = 0.2426153598589023
     avg_pval = 0
     for i in range(5):
-        avg_pval += EnrichmentRunner._calc_randomization_pval(500, 1, np.random.random(10000) < 0.1, 100000, 0.11)
+        avg_pval += EnrichmentRunner._calc_randomization_pval(500, 1, np.random.random(10000) < 0.1, 10000, 0.11)
     avg_pval /= 5
     assert np.isclose(avg_pval, hypergeom_pval, atol=0.02)
 
@@ -227,7 +224,7 @@ def test_classic_pvals(monkeypatch):
         def __getitem__(self, item):
             return self.dummy_node
 
-    monkeypatch.setattr(io, 'fetch_go_basic', lambda: DummyDAGTree())
+    monkeypatch.setattr(ontology, 'fetch_go_basic', lambda: DummyDAGTree())
 
     e = GOEnrichmentRunner(gene_set, 'elegans', 'WBGene', 0.05, 'classic', 'any', 'any', None, 'any', None, 'any', None,
                            False, False, '', False, False, False, '', False, 'hypergeometric', 'all')
@@ -247,7 +244,7 @@ def test_elim_pvals(monkeypatch):
     truth = pd.read_csv('tests/test_files/go_pvalues_elim_truth.csv', index_col=0).sort_index()
     with open('tests/test_files/obo_for_go_tests.obo', 'r') as f:
         dag_tree = ontology.DAGTree(f, ['is_a'])
-    monkeypatch.setattr(io, 'fetch_go_basic', lambda: dag_tree)
+    monkeypatch.setattr(ontology, 'fetch_go_basic', lambda: dag_tree)
 
     e = GOEnrichmentRunner(gene_set, 'elegans', 'WBGene', threshold, 'classic', 'any', 'any', None, 'any', None, 'any',
                            None, False, False, '', False, False, False, '', False, 'hypergeometric', 'all')
@@ -269,7 +266,7 @@ def test_weight_pvals(monkeypatch):
     with open('tests/test_files/obo_for_go_tests.obo', 'r') as f:
         dag_tree = ontology.DAGTree(f, ['is_a'])
 
-    monkeypatch.setattr(io, 'fetch_go_basic', lambda: dag_tree)
+    monkeypatch.setattr(ontology, 'fetch_go_basic', lambda: dag_tree)
 
     e = GOEnrichmentRunner(gene_set, 'elegans', 'WBGene', 0.05, 'classic', 'any', 'any', None, 'any', None, 'any', None,
                            False, False, '', False, False, False, '', False, 'hypergeometric', 'all')
@@ -290,7 +287,7 @@ def test_allm_pvals(monkeypatch):
     truth = pd.read_csv('tests/test_files/go_pvalues_allm_truth.csv', index_col=0).sort_index()
     with open('tests/test_files/obo_for_go_tests.obo', 'r') as f:
         dag_tree = ontology.DAGTree(f, ['is_a'])
-    monkeypatch.setattr(io, 'fetch_go_basic', lambda: dag_tree)
+    monkeypatch.setattr(ontology, 'fetch_go_basic', lambda: dag_tree)
 
     e = GOEnrichmentRunner(gene_set, 'elegans', 'WBGene', threshold, 'classic', 'any', 'any', None, 'any', None, 'any',
                            None, False, False, '', False, False, False, '', False, 'hypergeometric', 'all')
@@ -418,9 +415,11 @@ def test_enrichment_runner_fisher_enrichment(monkeypatch, params, truth):
     assert runner._fisher_enrichment('attribute') == truth
 
 
-def test_enrichment_runner_update_gene_set():
+@pytest.mark.parametrize('exclude_unannotated', [True, False])
+def test_enrichment_runner_update_gene_set(exclude_unannotated):
     runner = EnrichmentRunner.__new__(EnrichmentRunner)
     runner.single_set = False
+    runner.exclude_unannotated_genes = exclude_unannotated
     runner.background_set = set(pd.read_csv('tests/test_files/attr_ref_table_for_tests.csv', index_col=0).index)
     runner.gene_set = {'WBGene00000019', 'WBGene00000041', 'WBGene00000106', 'WBGene00001133', 'WBGene00003915',
                        'WBGene99991111'}
@@ -441,6 +440,7 @@ def test_enrichment_runner_update_gene_set_single_list(monkeypatch):
     assert runner.gene_set == updated_gene_set_truth
 
 
+@pytest.mark.parametrize('exclude_unannotated', [True, False])
 @pytest.mark.parametrize('save_csv,', [True, False])
 @pytest.mark.parametrize('return_nonsignificant,', [True, False])
 @pytest.mark.parametrize('fname', ['fname', None])
@@ -450,13 +450,13 @@ def test_enrichment_runner_update_gene_set_single_list(monkeypatch):
      (False, {'WBGene00000001', 'WBGene00000002'}, 'protein_coding', 'randomization',
       {'WBGene00000001', 'WBGene00000002', 'EBGene00000003'},
       'path/to/biotype/ref', 42, {'reps': 10000})])
-def test_enrichment_runner_api(return_nonsignificant, save_csv, fname, single_list, genes, biotypes, pval_func,
-                               background_set, biotype_ref_path, random_seed, kwargs, monkeypatch):
+def test_enrichment_runner_api(exclude_unannotated, return_nonsignificant, save_csv, fname, single_list, genes,
+                               biotypes, pval_func, background_set, biotype_ref_path, random_seed, kwargs, monkeypatch):
     monkeypatch.setattr('builtins.input', lambda x: 'fname')
 
     runner = EnrichmentRunner(genes, ['attr1', 'attr2'], 0.05, 'path/to/attr/ref', return_nonsignificant, save_csv,
                               fname, False, False, 'set_name', False, pval_func, biotypes,
-                              background_set, biotype_ref_path, single_list, random_seed, **kwargs)
+                              background_set, biotype_ref_path, exclude_unannotated, single_list, random_seed, **kwargs)
     if save_csv:
         assert runner.fname == 'fname'
     else:
@@ -640,12 +640,15 @@ def test_enrichment_runner_calculate_enrichment(monkeypatch):
     monkeypatch.setattr(EnrichmentRunner, '_calculate_enrichment_serial', lambda self: 'serial')
 
     runner = EnrichmentRunner.__new__(EnrichmentRunner)
-    runner.parallel = True
-    assert runner.calculate_enrichment() == 'parallel'
-    assert random_seed_status[0]
+    runner.attributes = list(range(10))
+
+    for backend in ['loky', 'threading', 'multiprocessing']:
+        runner.parallel_backend = backend
+        assert runner.calculate_enrichment() == 'parallel'
+        assert random_seed_status[0]
 
     random_seed_status[0] = False
-    runner.parallel = False
+    runner.parallel_backend = 'sequential'
     assert runner.calculate_enrichment() == 'serial'
     assert random_seed_status[0]
 
@@ -672,6 +675,7 @@ def test_enrichment_runner_set_random_seed(seed, is_legal):
 
 def _test_enrichment_runner_calculate_enrichment_get_constants():
     runner = EnrichmentRunner.__new__(EnrichmentRunner)
+    runner.parallel_backend = 'loky'
     runner.attributes = ['attribute1', 'attribute2', 'attribute4']
     runner.pvalue_kwargs = {'arg1': 'val1', 'arg2': 'val2'}
 
@@ -883,17 +887,19 @@ def test_noncategorical_enrichment_runner_enrichment_histogram(plot_style, plot_
     assert isinstance(res, plt.Figure)
 
 
+@pytest.mark.parametrize('exclude_unannotated', [True, False])
 @pytest.mark.parametrize('single_list,genes,biotypes,pval_func,background_set,biotype_ref_path, random_seed,kwargs',
                          [(True, np.array(['WBGene1', 'WBGene2'], dtype=str), None, 'xlmhg', None, None, None, {}),
                           (False, {'WBGene00000001', 'WBGene00000002'}, 'protein_coding', 'randomization',
                            {'WBGene00000001', 'WBGene00000002', 'EBGene00000003'},
                            'path/to/biotype/ref', 42, {'reps': 10000})])
 def test_go_enrichment_runner_api(monkeypatch, single_list, genes, biotypes, pval_func, background_set,
-                                  biotype_ref_path, random_seed, kwargs):
-    monkeypatch.setattr(io, 'fetch_go_basic', lambda: 'dag_tree')
+                                  biotype_ref_path, random_seed, kwargs, exclude_unannotated):
+    monkeypatch.setattr(ontology, 'fetch_go_basic', lambda: 'dag_tree')
     runner = GOEnrichmentRunner(genes, 'organism', 'gene_id_type', 0.05, 'elim', 'any', 'any', None, 'any', None, 'any',
                                 None, False, False, 'fname', False, False, False, 'set_name', False, pval_func,
-                                biotypes, background_set, biotype_ref_path, single_list, random_seed, **kwargs)
+                                biotypes, background_set, biotype_ref_path, exclude_unannotated, single_list,
+                                random_seed, **kwargs)
     if pval_func.lower() == 'xlmhg' and not does_python_version_support_single_set():
         assert runner.enrichment_func == False
         return
@@ -1448,6 +1454,7 @@ def test_go_enrichment_runner_go_elim_pvalues_parallel(monkeypatch, aspects):
 
     runner = GOEnrichmentRunner.__new__(GOEnrichmentRunner)
     runner.dag_tree = FakeDAG(aspects)
+    runner.parallel_backend = 'sequential'
     assert runner._go_elim_pvalues_parallel() == {aspect: 'success' for aspect in aspects}
 
 
@@ -1486,6 +1493,7 @@ def test_go_enrichment_runner_go_weight_pvalues_parallel(monkeypatch, aspects):
 
     runner = GOEnrichmentRunner.__new__(GOEnrichmentRunner)
     runner.dag_tree = FakeDAG(aspects)
+    runner.parallel_backend = 'sequential'
     assert runner._go_weight_pvalues_parallel() == {aspect: 'success' for aspect in aspects}
 
 
@@ -1708,6 +1716,7 @@ def test_go_enrichment_runner_parallel_over_grouping(monkeypatch, grouping, inds
         return {obj: ind for obj in group}
 
     runner = GOEnrichmentRunner.__new__(GOEnrichmentRunner)
+    runner.parallel_backend = 'loky'
     res = runner._parallel_over_grouping(my_func, grouping, inds)
     assert res == truth
 
@@ -1729,17 +1738,19 @@ def test_enrichment_runner_extract_xlmhg_results(pval_fwd, pval_rev, escore_fwd,
     assert log2escore == log2escore_truth
 
 
-@pytest.mark.parametrize('single_list,genes,biotypes,pval_func,background_set,biotype_ref_path, random_seed,kwargs',
-                         [(True, np.array(['WBGene1', 'WBGene2'], dtype=str), None, 'xlmhg', None, None, None, {}),
-                          (False, {'WBGene00000001', 'WBGene00000002'}, 'protein_coding', 'randomization',
-                           {'WBGene00000001', 'WBGene00000002', 'EBGene00000003'},
-                           'path/to/biotype/ref', 42, {'reps': 10000})])
+@pytest.mark.parametrize('exclude_unannotated', [True, False])
+@pytest.mark.parametrize(
+    'single_list,genes,biotypes,pval_func,background_set,biotype_ref_path, random_seed,graph_format,kwargs',
+    [(True, np.array(['WBGene1', 'WBGene2'], dtype=str), None, 'xlmhg', None, None, None, 'none', {}),
+     (False, {'WBGene00000001', 'WBGene00000002'}, 'protein_coding', 'randomization',
+      {'WBGene00000001', 'WBGene00000002', 'EBGene00000003'},
+      'path/to/biotype/ref', 42, 'pdf', {'reps': 10000})])
 def test_kegg_enrichment_runner_api(monkeypatch, single_list, genes, biotypes, pval_func, background_set,
-                                    biotype_ref_path, random_seed, kwargs):
+                                    biotype_ref_path, random_seed, graph_format, kwargs, exclude_unannotated):
     monkeypatch.setattr(KEGGEnrichmentRunner, 'get_taxon_id', lambda *args: ('a', 'b'))
-    runner = KEGGEnrichmentRunner(genes, 'organism', 'gene_id_type', 0.05, True, False, 'fname', False, False,
-                                  'set_name', False, pval_func,
-                                  biotypes, background_set, biotype_ref_path, single_list, random_seed, **kwargs)
+    runner = KEGGEnrichmentRunner(genes, 'organism', 'gene_id_type', 0.05, True, False, 'fname', False, False, True,
+                                  'set_name', False, pval_func, biotypes, background_set, biotype_ref_path,
+                                  exclude_unannotated, single_list, random_seed, graph_format, **kwargs)
 
 
 @pytest.mark.parametrize('got_gene_id_type', (True, False))
@@ -1906,10 +1917,31 @@ def test_kegg_enrichment_runner_fetch_attributes():
     assert runner.attributes_set == truth_attributes_set
 
 
-def test_dag_plot_for_namespace():
-    runner = GOEnrichmentRunner.__new__(GOEnrichmentRunner)
-    runner.results = io.load_csv('tests/test_files/go_enrichment_runner_sample_results.csv', index_col=0)
-    runner.dag_tree = io.fetch_go_basic()
-    runner.en_score_col = 'colName'
-    runner.ontology_graph_format = 'png'
-    runner._dag_plot_for_namespace('biological_process', 'title', 'ylabel', 50)
+@pytest.mark.parametrize('single_set,graph_format', [(True, 'png'), (False, 'none')])
+def test_kegg_enrichment_runner_pathway_plot(single_set, graph_format, monkeypatch):
+    pathway_id_truth = 'pth_id'
+    translator_truth = 'translator obj'
+    plotted = []
+
+    class MockPathway:
+        def __init__(self, pathway_id, translator):
+            assert pathway_id == pathway_id_truth
+            assert translator == translator_truth
+
+        def plot_pathway(self, significant, ylabel, graph_format):
+            assert significant == runner.gene_set
+            if single_set:
+                assert ylabel == KEGGEnrichmentRunner.SINGLE_SET_ENRICHMENT_SCORE_YLABEL
+            else:
+                assert ylabel == KEGGEnrichmentRunner.ENRICHMENT_SCORE_YLABEL
+            assert graph_format == graph_format
+            plotted.append(True)
+
+    monkeypatch.setattr(ontology, 'fetch_kegg_pathway', lambda *args, **kwargs: MockPathway(*args, **kwargs))
+
+    runner = KEGGEnrichmentRunner.__new__(KEGGEnrichmentRunner)
+    runner.gene_set = {'a', 'b', 'c'}
+    runner.single_set = single_set
+    runner.pathway_graphs_format = graph_format
+    runner.gene_id_translator = translator_truth
+    runner.pathway_plot(pathway_id_truth)
