@@ -4,6 +4,10 @@ import rnalysis.utils.io
 from rnalysis.utils import io
 from rnalysis.utils.io import *
 from rnalysis.utils.io import _format_ids_iter, _ensmbl_lookup_post_request
+from tests import is_uniprot_available, is_ensembl_available
+
+ENSEMBL_AVAILABLE = is_ensembl_available()
+UNIPROT_AVAILABLE = is_uniprot_available()
 
 
 class MockResponse(object):
@@ -394,6 +398,7 @@ def test_infer_taxon_from_gene_ids_no_species(monkeypatch):
         infer_taxon_from_gene_ids([])
 
 
+@pytest.mark.skipif(not UNIPROT_AVAILABLE, reason='UniProt REST API is not available at the moment')
 def test_map_gene_ids_connectivity():
     ids_uniprot = ['P34544', 'Q27395', 'P12844']
     ids_wormbase = ['WBGene00019883', 'WBGene00023497', 'WBGene00003515']
@@ -432,6 +437,7 @@ def test_map_gene_ids_to_same_set(id_type):
         assert mapper[i] == i
 
 
+@pytest.mark.skipif(not UNIPROT_AVAILABLE, reason='UniProt REST API is not available at the moment')
 @pytest.mark.parametrize('ids,map_from,map_to,req_from,req_to,req_query,txt,truth',
                          [(['P34544', 'Q27395', 'P12844'], 'UniProtKB', 'WormBase', 'ACC', 'WORMBASE_ID',
                            'P34544 Q27395 P12844',
@@ -439,7 +445,11 @@ def test_map_gene_ids_to_same_set(id_type):
                            {'P34544': 'WBGene00019883', 'Q27395': 'WBGene00023497', 'P12844': 'WBGene00003515'}
                            )])
 def test_map_gene_ids_request(monkeypatch, ids, map_from, map_to, req_from, req_to, req_query, txt, truth):
-    def mock_get(url, params):
+    legal_types = get_legal_gene_id_types()
+
+    def mock_get(url, params=None):
+        if params is None:
+            return
         assert url == 'https://www.uniprot.org/uploadlists/'
         assert params == {'from': req_from,
                           'to': req_to,
@@ -449,6 +459,7 @@ def test_map_gene_ids_request(monkeypatch, ids, map_from, map_to, req_from, req_
         return MockResponse(text=txt)
 
     monkeypatch.setattr(requests, 'get', mock_get)
+    monkeypatch.setattr(rnalysis.utils.io, 'get_legal_gene_id_types', lambda: legal_types)
     res = map_gene_ids(ids, map_from, map_to)
     for gene_id in truth:
         assert res[gene_id] == truth[gene_id]
@@ -790,36 +801,6 @@ def test_kegg_annotation_iterator_get_taxon_tree_cached(monkeypatch):
     assert KEGGAnnotationIterator._get_taxon_tree() == truth
 
 
-def test_get_gui_videos(monkeypatch):
-    mock_dir = Path('tests/.pytest_cache/video_test_cache')
-    video_files = ('first_vid.webp', 'second_vid.webp', 'third_vid.webp')
-    visited = []
-
-    def mock_get(url, params):
-        base_url = 'https://github.com/GuyTeichman/RNAlysis/blob/master/rnalysis/gui/videos/'
-        assert url.startswith(base_url)
-        visited.append(url.split(base_url)[1])
-        return MockResponse(content='content')
-
-    monkeypatch.setattr(requests, 'get', mock_get)
-    monkeypatch.setattr(rnalysis.utils.io, 'get_tutorial_videos_dir',
-                        lambda: mock_dir)
-
-    try:
-        list(get_gui_videos(video_files))
-        assert sorted(visited) == ['first_vid.webp', 'second_vid.webp', 'third_vid.webp']
-        assert mock_dir.exists()
-        file_count = 0
-        for file in mock_dir.iterdir():
-            assert file.name in visited
-            with open(file) as f:
-                assert f.read() == 'content'
-            file_count += 1
-        assert file_count == len(video_files)
-    finally:
-        shutil.rmtree(mock_dir, ignore_errors=True)
-
-
 class MockProcess:
     def __init__(self, returncode: int):
         self.stdout = [b'things', b'to', b'print']
@@ -874,51 +855,3 @@ def test_is_rnalysis_outdated(monkeypatch, this_version, response, expected):
     monkeypatch.setattr(requests, 'get', lambda *args, **kwargs: response)
     monkeypatch.setattr(rnalysis.utils.io, '__version__', this_version)
     assert is_rnalysis_outdated() == expected
-
-
-@pytest.mark.parametrize('feature_type,truth', [
-    ('gene', {
-        'ENSG00000168671': 'protein_coding',
-        'ENSG00000249641': 'antisense_RNA',
-        'ENSG00000123364': 'protein_coding',
-        'ENSG00000123407': 'protein_coding',
-        'ENSG00000123388': 'protein_coding',
-        'ENSG00000180818': 'protein_coding',
-        'ENSG00000197757': 'protein_coding',
-        'ENSG00000180806': 'protein_coding',
-        'ENSG00000037965': 'protein_coding',
-        'ENSG00000198353': 'protein_coding',
-        'ENSG00000172789': 'other'}),
-    ('transcript', {
-        "ENST00000282507": "protein_coding",
-        "ENST00000513300": "protein_coding",
-        "ENST00000504685": "nonsense_mediated_decay",
-        "ENST00000504954": "processed_transcript",
-        "ENST00000515131": "protein_coding",
-        "ENST00000512916": "antisense_RNA",
-        "ENST00000243056": "protein_coding",
-        "ENST00000243103": "protein_coding",
-        "ENST00000546378": "protein_coding",
-        "ENST00000243082": "protein_coding",
-        "ENST00000515593": "protein_coding",
-        "ENST00000303460": "protein_coding",
-        "ENST00000511575": "processed_transcript",
-        "ENST00000514415": "processed_transcript",
-        "ENST00000504315": "protein_coding",
-        "ENST00000509328": "protein_coding",
-        "ENST00000394331": "protein_coding",
-        "ENST00000243108": "protein_coding",
-        "ENST00000513413": "processed_transcript",
-        "ENST00000504557": "processed_transcript",
-        "ENST00000508190": "protein_coding",
-        "ENST00000303450": "protein_coding",
-        "ENST00000040584": "protein_coding",
-        "ENST00000303406": "protein_coding",
-        "ENST00000507650": "processed_transcript",
-        "ENST00000430889": "protein_coding",
-        "ENST00000312492": "other", })
-])
-def test_map_gene_to_attr(feature_type, truth):
-    gtf_path = 'tests/test_files/kallisto_tests/transcripts.gtf'
-    res = map_gene_to_attr(gtf_path, feature_type + '_biotype', feature_type, False, False, False)
-    assert res == truth
